Module: SVG

Defined in:
lib/statsample/graph/svggraph.rb

Overview

replaces all key and fill classes with similar ones, without opacity this allows rendering of svg and png on rox and gqview without problems

Defined Under Namespace

Modules: Graph