Method: Statsample::Vector#+

Defined in:
lib/statsample/vector.rb

#+(v) ⇒ Object

Vector sum.

  • If v is a scalar, add this value to all elements

  • If v is a Array or a Vector, should be of the same size of this vector every item of this vector will be added to the value of the item at the same position on the other vector



437
438
439
# File 'lib/statsample/vector.rb', line 437

def +(v)
_vector_ari("+",v)
end