Method: Statsample::Codification.create_yaml
- Defined in:
- lib/statsample/codification.rb
.create_yaml(dataset, vectors, io = nil, sep = Statsample::SPLIT_TOKEN) ⇒ Object
Create a yaml to create a dictionary, based on vectors The keys will be vectors name on dataset and the values will be hashes, with keys = values, for recodification
v1=%w{a,b b,c d}.to_vector
ds={"v1"=>v1}.to_dataset
Statsample::Codification.create_yaml(ds,['v1'])
=> "--- \nv1: \n a: a\n b: b\n c: c\n d: d\n"
56 57 58 59 |
# File 'lib/statsample/codification.rb', line 56 def create_yaml(dataset, vectors, io=nil, sep=Statsample::SPLIT_TOKEN) pro_hash=create_hash(dataset, vectors, sep) YAML.dump(pro_hash,io) end |