Method: Statsample::Vector#-
- Defined in:
- lib/statsample/vector.rb
#-(v) ⇒ Object
Vector rest.
-
If v is a scalar, rest this value to all elements
-
If v is a Array or a Vector, should be of the same size of this vector every item of this vector will be rested to the value of the item at the same position on the other vector
447 448 449 |
# File 'lib/statsample/vector.rb', line 447 def -(v) _vector_ari("-",v) end |